//===============================================================================
// Name:     wwgmap.js
// Purpose:  Create WaterWatch Google Maps.
// Date:     2010-03-09
// Version:  1.1.5
// Author:   X. Jian
//===============================================================================
var xgtool = {
	'ver'    : '1.1.0',
	'vdt'    : '2010-03-01'
};
//==============================================================================
// Name:    isArray
// Purpose: Test whether an input is an array.
// Usage:   bolean xarray.isArray(arg);
//===============================================================================
xgtool.isArray = function() {
	//alert("isArray(): ver = " + this.ver);
	if (arguments[0] == null) return false;
	if (typeof arguments[0] == 'object') {  
		var tmp = arguments[0].constructor.toString().match(/array/i); 
 		return (tmp != null);  
	}
	return false;
}

//function gmap_click(overlay, point) {
//	map_nm = gmap.getCurrentMapType().getName();
//}
//
//==============================================================================
// Name:    xgload
// Purpose: Load Google Map api
// Usage:   xgload(xml_file, ygopts)
// Arguments:
//		ygopts -- A JSON object of google-map options: ygopts = {'opt': value, ...}
//              'opt' can one of the followings:
//			'cc_lat' -- latitude of center of the map.
//			'cc_lng' -- longtitude of center of the map.
//			'sw_lat' -- latitude of center of the map.
//			'ne_lng' -- longtitude of center of the map.
//			'sw_lat' -- latitude of center of the map.
//			'ne_lng' -- longtitude of center of the map.
//			'dot_dir' -- Where is the 'dots' directory.
//			'zoom_level' -- Zoom level. 
//				1. Default is 7. 
//				2. If 'sw_lat', 'sw_lng', 'ne_lat', and 'ne_lng' are specified, 'zoom_level' 
//					will not be used. Instead, a zoom_level will be determined by 'SW' and 'NE' 
//					coordinates.
//			'popup' -- An indicator to have an 'info' window popup while on-clicking.
//				1. Default is 1.
//          2. If not, on-click will open 'link' url.
//===============================================================================
function xgload(xml_file, ygopts) {
   if (!GBrowserIsCompatible()) {return 0;}
	//
	//--------Set 'default' options.
	//
	xgopts = {
		'cc_lat': 38.61, 
		'cc_lng': -97.95, 
		'zoom_level':7,
		'popup': 1,
		'dot_dir': 'wwgmaps/images',
		'mt': 'real'
	};
	if (null != ygopts) {
		for (var s in ygopts) {
			xgopts[s] = ygopts[s];
		}
	}
	//
	//----------check arguments.
	//
	//zoom_level = 7;
	//cc_lat = xopts.cc_lat;
	//cc_lng = xopts.cc_lng;
	if (xgopts.sw_lat && xgopts.sw_lng && xgopts.ne_lat && xgopts.ne_lng) {
		xgopts.zoom_level = 0;
	}
	//alert('dot_dir = ' + xgopts.dot_dir);
	//return '';
	//
	//----------Create an array to list of 'possible' background maps.
	//
   var mapSpecs = new Array();
   mapSpecs.push(G_NORMAL_MAP);
   mapSpecs.push(G_SATELLITE_MAP);
   mapSpecs.push(G_HYBRID_MAP);
   mapSpecs.push(G_PHYSICAL_MAP);
	//
	//----------Create 'Google-Map' object: GMap2 and add 'zooming' controls.
	//
   gmap = new GMap2(document.getElementById("map"), { mapTypes: mapSpecs } );
   gmap.addControl(new GMapTypeControl());
   gmap.addControl(new GLargeMapControl());
   gmap.addControl(new GScaleControl());
   
   

   var xgpoint = null;
	var xgoverlay = null;
   var xgzooms = new Array();
	var xgzoom = null;
	//gmap.disableDoubleClickZoom();
   //var gmap_click = '';




function makeIcon(url, w, h, anchor_x, anchor_y ){
   if (url) {
      gicon = new GIcon();
	   gicon.image = url;
	}
	else {
	   gicon = new GIcon(G_DEFAULT_ICON);
	}
   gicon.iconSize = new GSize(w, h);
   gicon.shadowSize = new GSize(0, 0);
   gicon.iconAnchor = new GPoint(anchor_x, anchor_y);
   gicon.infoWindowAnchor = new GPoint(anchor_x, anchor_y);
	return gicon;
}
//
function setMrkZIndex(marker, b) {
	return GOverlay.getZIndex(marker.getPoint().lat()) + marker.zidx * 100000;
}
function makeMarker(point, opts) {
   var name = opts.name;
	var html = opts.html;
	var icontype = opts.icontype;
	var labels = opts.labels;
	var marker;
	if (typeof(opts.zidx) != 'undefined') {
      marker = new GMarker(point, gicons[icontype],{zIndexProcess:setMrkZIndex});	
	   marker.zidx = opts.zidx;
	}
	else {
      marker = new GMarker(point, gicons[icontype]);
	}
   GEvent.addListener(marker, "click", function() {
   if (xgtool.isArray(html)) {
		//alert('isArray: ' + xgtool.isArray(html) + ', length: ' + html.length);
		//
		//------------Set 'minimum' width of popup if necessary
		//
		var w = 0;
	   if (html.length >= 2) {
		   var w2 = html.length*88;
			if (w2 > w) {w = w2}
		}
		//alert('popup width: ' + w);
		if (w > 0 || h > 0) {
         html[0] = '<div style="width:'+w+'px;0">' + html[0] + '<\/div>';
      }
		
      var tabs = [];
		//tabs.push(new GInfoWindowTab(labels[0],html[0]));
      for (var i=0; i<html.length; i++) {
          tabs.push(new GInfoWindowTab(labels[i],"<center>" + html[i] + "</center>"));
      }
      marker.openInfoWindowTabsHtml(tabs);		
	}
   else if (html) {
   	marker.openInfoWindowHtml(html); 
	}
	xgpoint = marker.getPoint(); 
});


	if (name) {
   	marker.tooltip = '<div class="sitetip">'+name+'</div>';
   	GEvent.addListener(marker,"mouseover", function() {showTooltip(marker);});
   	GEvent.addListener(marker,"mouseout",  function() {tooltip.style.visibility="hidden"});
	}

   //alert(marker + ', tip = ' + marker.zidx);
	//exit;
   addToMap(marker);
   return marker;
}
//
function createMarker(point,name,html,icontype,labels) {
   var marker = new GMarker(point, gicons[icontype]);
   GEvent.addListener(marker, "click", function() {
   if (xgtool.isArray(html)) { // html = array(tab_0, tabl_1, ..., tab_n);
		//alert('isArray: ' + xgtool.isArray(html) + ', length: ' + html.length);
		//
		//------------Set 'minimum' width of popup if necessary
		//
		var w = 0;
	   if (html.length >= 2) {
		   var w2 = html.length*88;
			if (w2 > w) {w = w2}
		}
		//alert('popup width: ' + w);
		if (w > 0 || h > 0) {
         html[0] = '<div style="width:'+w+'px;0">' + html[0] + '<\/div>';
      }
		
      var tabs = [];
		//tabs.push(new GInfoWindowTab(labels[0],html[0]));
      for (var i=0; i<html.length; i++) {
          tabs.push(new GInfoWindowTab(labels[i],"<center>" + html[i] + "</center>"));
      }
      marker.openInfoWindowTabsHtml(tabs);		
	}
	else if (html.match(/^http\S+/)) {
		window.location.href = html;
		return marker;
	}
   else if (html) {
   	marker.openInfoWindowHtml(html); 
	}
	xgpoint = marker.getPoint(); 
});


	if (name) {
   	marker.tooltip = '<div class="sitetip">'+name+'</div>';
   	GEvent.addListener(marker,"mouseover", function() {showTooltip(marker);});
   	GEvent.addListener(marker,"mouseout",  function() {tooltip.style.visibility="hidden"});
	}

   addToMap(marker);
   return marker;
}
function showTooltip(marker) {
   tooltip.innerHTML = marker.tooltip;
	var point=gmap.getCurrentMapType().getProjection().fromLatLngToPixel(gmap.getBounds().getSouthWest(),gmap.getZoom());
	var offset=gmap.getCurrentMapType().getProjection().fromLatLngToPixel(marker.getPoint(),gmap.getZoom());
	var anchor=marker.getIcon().iconAnchor;
	var width=marker.getIcon().iconSize.width;
	var pos = new GControlPosition(G_ANCHOR_BOTTOM_LEFT, new GSize(offset.x - point.x - anchor.x + width,- offset.y + point.y +anchor.y));
	pos.apply(tooltip);
	tooltip.style.visibility="visible";
}

function addToMap(marker) {
	gmap.addOverlay(marker);
}
   var icon = new GIcon();
   icon.infoWindowAnchor = new GPoint(5, 1);
   var tooltip = document.createElement("div");
   document.getElementById("map").appendChild(tooltip);
   tooltip.style.visibility="hidden";


//===============================================================================
// Name: xml_open
// Purpose: Open a XML file
//===============================================================================
function xml_open(xml_file, itag_nms, fidx, popup_as_table) {
	//alert(xml_file + "popup_as_table = " + popup_as_table);
	if (fidx == null) {
		fidx = 0;
	}
	//
	//--------Set default xml tag name
	//
	//if (null == tag_nms) {
	tag_nms = {
		lat: 'lat',
		lng: 'lng',
		tip: 'tip',
		popup: 'popup',
		icon: 'icon',
		symbol: 'symbol',
		url: 'url',
		marker: 'marker',
		name: 'name'
	};
	//}
	//
	//------------Update 'xml' tag names
	//
	if (null != itag_nms) {
		for (var nm in itag_nms) {
			tag_nms[nm] = itag_nms[nm];
		}
	}
	//
	//------------Create 'AJAX' object
	//
	var request = GXmlHttp.create();
   request.open("GET", xml_file, true);
   request.onreadystatechange = function() {
   	if (request.readyState == 4) {
		   xml_parse_n_map(request.responseText, tag_nms);
		}
   }
	request.send(null);

function xml_parse_n_map(data, tag_nms) {
	//
	//-------Parses a given string as XML text and returns a DOM representation. If the browser doesn't 
	//       support XML parsing natively, this returns the DOM node of an empty DIV  element.
	//
   var xmlDoc = GXml.parse(data);
   //
   //------obtain the array of markers and loop through it
	//
	var marker_tag = tag_nms['marker'];
	//alert('marker_tag' + marker_tag);
   var markers = xmlDoc.documentElement.getElementsByTagName(marker_tag);  
	//alert("no. of markers = " + markers.length);
	if (markers != null) {xml_marker(markers, tag_nms);} 
	//
	//------------draw 'polyline' 
   //var lines = xmlDoc.documentElement.getElementsByTagName("line");
	//if (lines != null) {xml_polyline(lines, tag_nms);}
	//
	//------------Draw 'polygons'
	//
	//var polygon_tag = tag_nms['polygon'];
	//var polygons = xmlDoc.documentElement.getElementsByTagName(polygon_tag);
	//if (polygons != null) {xml_polygon(polygons, tag_nms);}
   return 1;
}
//==================================================================
// Name:    xml_marker
// Purpose: Create markers from
//================================================================== 
function xml_marker(markers, tag_nms) {
	//alert("xml_marker: popup_as_table = " + popup_as_table);
	for (var i = 0; i < markers.length; i++) {
		//
      //--------obtain the attribues of each marker
		//
		tag_vas = {};
		for (var nm in tag_nms) {
			tag_nm = tag_nms[nm];
			//alert(nm + ', getting attribute = ...');
         tag_vas[nm] = markers[i].getAttribute(tag_nm);
			//alert(nm + ', attribute = ' + va);
			if (null == tag_vas[nm]) {
			 	tag_vas[nm] = GXml.value(markers[i].getElementsByTagName(tag_nm)[0]);
				if (tag_vas[nm] == null) {tab_vas[nm] = '';}
			}
			//tag_vas[nm] = va;
			//alert("xinfos: " + nm  + " = " + xinfos[nm]);
		}
		//
		//--------------Make sure that icon is specpfied. If not, create an icon.
		//
		//alert("file: " + fidx + ": 0. icon = " + tag_vas['icon'] + ", symbol = " + tag_vas['symbol']);
		if (!tag_vas['icon'] && tag_vas['symbol'].length) {
			sym = (tag_vas['symbol'].length == 1)? "0" + tag_vas['symbol'] : tag_vas['symbol'];
			tag_vas['icon'] = "mrk_" + sym + "_" + fidx;
		}
		//alert("file: " + fidx + ": n. icon = " + tag_vas['icon'] + ", symbol = " + tag_vas['symbol'] + ", lat,lng = " + tag_vas['lat'] + "," + tag_vas['lng'] );

		//alert("xinfos = " + tag_vas['tip'] + "," + tag_vas['popup'] + "," + tag_vas['icon']);
		point = new GPoint(parseFloat(tag_vas['lng']), parseFloat(tag_vas['lat']));
		//
		strTip = tag_vas['tip'];
		//
		//-------Split popup into a table
		//
		popup_as_table = true;
		strPopups = '';
		strLabels = '';
		strInfos = tag_vas['popup'].split("!!");
		//alert(i + " strinfos = " + strInfos[0] + ', popup = ' + xopts.nopopup);
		if (strInfos.length >= 2) {
		   strPopups = [];
			strLabels = [];
		   for (var j = 0; j < strInfos.length; j++) {
				vas = strInfos[j].split('??');
				if (vas.length == 1) {
					strPopups[j]  = vas[0];
					strLabels[j] = 'Tab ' + j;
				}
				else {
				  strLabels[j] = vas[0];
				  //alert("0, j = " + j);
				  //
				  //--------------Get 'tab' contents
				  //
				  if (vas.length >= 3) {
				     if (vas[2] == 'image') {
					     va = "<img src='" + vas[1] + "' alt='image' border='0' />";
					  }
					  else {
					     va = vas[1];
					  }
              		//
				  		//------------Are there any links?
						//
				  		if (vas.length >=4) {
							va = "<a href='" + vas[3] + "' target='_blank'>" + va + "</a>";
				  		} 
				  }
				  else {
				  	  va = (popup_as_table)? str2table(vas[1], tag_vas['tip'], tag_vas['url']) : vas[1];
					  //alert('va: ' + va);
					  //va=vas[1];
				  }
				  //alert("n, j = " + j);
				  //alert("va: " + va + ", legnth = " + vas.length);
				  strPopups[j] = va;
				  //alert(i + "," + j + ", length = " + strPopups.length);
				}
			}
		}
		else {
			// There is no popup information specified.
			//	If there is a link, use link for click event.
			//
			if (strInfos[0] == "") { 
				strPopups = tag_vas['url']; //Set 'popup' string as a url to link. See 'createMarker()' function
				if (tag_vas['name']) {
					strTip = tag_vas['name'] + ', ' + strTip;
				}
			}
			else {
				strPopups = (popup_as_table)? str2table(strInfos[0], tag_vas['tip'], tag_vas['url']) : strInfos[0];
			}
	   }
		//
		//--------Create an marker
		//
		//if (xgtool.isArray(strPopups)) {alert(i + 'no. of tabs = ' + strPopups.length)}
		//marker = createMarker(point,tag_vas['tip'],tag_vas['popup'],tag_vas['icon']);
		marker = createMarker(point,strTip,strPopups,tag_vas['icon'], strLabels);
	}
	return;
}
//=====================================================================================
function str2table(str, title, title_url) {
	var items = str.split(";");
	strPopup="<div class='sitepopup'>";
	if (null != title) {
	   //strPopup +=(title_url)? "<table><caption><a href='" + title_url + "'>" + title + "</a></caption>"
	   //                     : "<table><caption>" + title + "</caption>";
	   strPopup +=(title_url)? "<a href='" + title_url + "'>" + title + "</a>" : title;
		strPopup += '<table>';
	}
	for (var j = 0; j < items.length; j++) {
		if (items[j] == "") {continue;}
		vas = items[j].split(':');
		if (vas.length == 1) {vas[1] = '';}
		else if(vas.length > 2) {vas[1] = items[j].replace(vas[0] + ":", '');}
		strPopup += "<tr><td>" + vas[0] + ":</td><td>" + vas[1] + "</td></tr>";
	}
	//strPopup += "<tr><td colspan='2'>powered by <a href='http://waterwatch.usgs.gov'><span style='font-weight: bold;'>USGS WaterWatch</span></a></td></tr>";
	strPopup += "</table>";
	strPopup += "<p style='text-align: center;'>powered by <a href='http://waterwatch.usgs.gov'><span style='font-weight: bold;'>USGS WaterWatch</span></a></p>";
	strPopup += '</div>';
	return strPopup;
}
} //end of 'xml_open()'

//alert('dot_dir = ' + dot_dir);
//
//-------------set icons
//
var gicons = [];
dot_dir = xgopts.dot_dir;
mt = (xgopts.mt == 'flood')? 'flood' : 'real';
gicons['mrk_00_0'] = makeIcon(dot_dir + '/b_' + mt + '00dot.gif', 10, 10, 5, 5)
gicons['mrk_01_0'] = makeIcon(dot_dir + '/b_' + mt + '01dot.gif', 10, 10, 5, 5)
gicons['mrk_02_0'] = makeIcon(dot_dir + '/b_' + mt + '02dot.gif', 10, 10, 5, 5)
gicons['mrk_03_0'] = makeIcon(dot_dir + '/b_' + mt + '03dot.gif', 10, 10, 5, 5)
gicons['mrk_04_0'] = makeIcon(dot_dir + '/b_' + mt + '04dot.gif', 10, 10, 5, 5)
gicons['mrk_05_0'] = makeIcon(dot_dir + '/b_' + mt + '05dot.gif', 10, 10, 5, 5)
gicons['mrk_06_0'] = makeIcon(dot_dir + '/b_' + mt + '06dot.gif', 10, 10, 5, 5)
gicons['mrk_07_0'] = makeIcon(dot_dir + '/b_' + mt + '07dot.gif', 10, 10, 5, 5)
gicons['mrk_08_0'] = makeIcon(dot_dir + '/b_' + mt + '08dot.gif', 10, 10, 5, 5) 
//alert('cc: ' + cc_lat + ',' + cc_lng);
if (xgopts.zoom_level > 1) {
	gmap.setCenter(new GLatLng(xgopts.cc_lat, xgopts.cc_lng), xgopts.zoom_level);
}
else {
	gmap.setCenter(new GLatLng(xgopts.cc_lat, xgopts.cc_lng));
	gmap.setZoom(gmap.getBoundsZoomLevel(new GLatLngBounds(new GLatLng(xgopts.sw_lat, xgopts.sw_lng),new GLatLng(xgopts.ne_lat,xgopts.ne_lng))));
}

gmap.setMapType(G_PHYSICAL_MAP)
 
GEvent.addListener(gmap, 'click', function(overlay, point) {
		//alert('gmap is clicking ...' + overlay + ', type: ' + overlay.type + ',' + point);
	   if (overlay) {
		   xgoverlay = overlay;
			if (overlay.type == 'poly') {
			   xgpoint = overlay.getBounds().getCenter();
				if (typeof overlay.onclick == 'function') {
				   overlay.onclick();
				}
			}
			else {
		      xgpoint = overlay.getPoint();
			}
		}
		else {
			xgpoint = point;
		}
		
		//alert('gmap is clicking ...' + overlay + ',' + point + ', xgpoint: ' + xgpoint);
			
		
});
GEvent.addListener(gmap, 'dblclick', function(overlay, point) {
			   //alert('dbould-clicking ... overlay = ' + overlay + ', point = ' + point);
				if (null == overlay){overlay = xgoverlay;}
				if (overlay && overlay.type == 'poly' && typeof overlay.ondblclick == 'function') {
					//alert('running ploy ondblclicking ...');
				   overlay.ondblclick();
				}
				else {
			   	gmap_dblclick(overlay, point); 
				}
			});
GEvent.addListener(gmap, 'zoomend', function(intZoomLevelOld, intZoomLevelNew) {
	  if (xgpoint) {gmap.setCenter(xgpoint);}
	  xgzooms.push(intZoomLevelNew);
});

if (!xgopts.popup) {
	tip_fld = 'flowinfo';
	popup_fld = '';
}
else {
	tip_fld = 'name';
	popup_fld = 'flowinfo';
}
//xml_open(xml_file, {lat: 'lat',lng: 'lng', popup: popup_fld, tip: tip_fld, icon: 'icon', symbol:'class',marker: 'site',markers: 'sites', polyline: 'polyline',polylines: 'polylnes',polygon: 'polygon',polygons: 'polygons',url: 'url'}, 0, false)
xml_open(xml_file, {popup: popup_fld, tip: tip_fld, symbol:'class', marker: 'site', markers: 'sites'}, 0, false)
}